//////////////////////////////////////////////////////////////////////
// DriverCfg.cpp - Device Manager Property Page for AFT adapters.
//
// Author	:	David Rokhvarg
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "sdlacfg.h"

#define front_end_cfg wandev_conf.fe_cfg

HKEY	hkey = 0;
GLOBALS Globals_Old_Cfg;
GLOBALS Globals_New_Cfg;
BOOL	okay = FALSE;

//driver cfg functions prototypes
BOOL DriverSaveCfgData(HWND hdlg);

int 
DriverOnNotify(
	HWND    ParentHwnd,
	LPNMHDR NmHdr,
	SETUPSTUFF * Params
	);
void DriverInit(HKEY hkey, HWND hdlg);

static LRESULT CALLBACK RestartingDriverDlg(HWND hdlg, UINT iMessage, WPARAM wParam, LPARAM lParam);

const look_up_t network_sync_options[] = {
	{"Off-default", 0},
	{"On", 1},
	{NULL,		0}
};

const look_up_t aft_tdmv_codec_options[] = {
	{"ALaw",	WAN_TDMV_ALAW},
	{"MuLaw",	WAN_TDMV_MULAW},
	{NULL,		0}
};

const look_up_t hwec_clksrc_options[] = {
	{"Yes",	WANOPT_YES},
	{"No",	WANOPT_NO},
	{NULL,		0}
};

static const look_up_t tx_tristate_mode[] = {
	{"Off (default)",	WANOPT_NO},
	{"On",				WANOPT_YES},
	{NULL,				0}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////

DLGPROC_RETURN_TYPE WINAPI DrvPageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{							// PageDlgProc
	switch (msg)
	{						// process message
	case WM_NOTIFY:
		Debug(DRV_CFG_PAGE, "Drv: WM_NOTIFY");

		//If there is a problem with the new configuration 
		//refuse to close the property page and return TRUE.
		//If all checks are OK allow the closing.
        if(DriverOnNotify(hdlg, (NMHDR *)lParam, stuff) == HANDLE_ERROR)
		{
			Debug(DRV_CFG_PAGE, "Invalid Driver Configuration, refusing to close!");
			SetWindowLong(hdlg, DWL_MSGRESULT, PSNRET_INVALID);
			return TRUE;
		}
		break;

	case WM_INITDIALOG:	//this message received only ONCE
		{						
			Debug(DRV_CFG_PAGE, "Drv:WM_INITDIALOG");

			// Open registry for our device
			// This is done in device cfg page since it is the first page to open.
			if (okay == TRUE)
			{	
				//using configuration from the registry, update visual controls
				DriverInit(hkey, hdlg);
			}

			if (!okay)
			{	// disable everything
				EnableWindow( GetDlgItem(hdlg, IDCHECK_IS_LOG_TO_FILE), FALSE);
			}
		}
		break;
		
   	case WM_DESTROY:
		{
			LPSTR infofile = (LPSTR) GetWindowLongPtr(hdlg, DWL_USER);
			if (infofile)
				GlobalFree((HGLOBAL) infofile);
		}
		break;

	case WM_SHOWWINDOW:
		//This message received each time the Dialog shown/hidden.
		//Update visual controls with latest information stored in "Globals_New_Cfg".
		Debug(DRV_CFG_PAGE, "Drv:WM_SHOWWINDOW");
		set_dialog_wanpipe_name(hdlg);
		DriverInit(hkey, hdlg);
		break;

    case WM_COMMAND:
		switch (LOWORD(wParam))
		{					// process command notification
		case BUTTON_HELP:
			WinHelp(GetActiveWindow(),"WanpipePnp.hlp",HELP_CONTEXT,IDH_DRIVER_CFG);
			return TRUE;

		case COMBO_WANPIPE_NUMBER:
			if (GET_WM_COMMAND_CMD(wParam, lParam) == CBN_SELCHANGE){
				//WanpipeNumber: ask user to confirm, cause it is a serious change.
				if(yes_no_question("Are you sure you want change the 'Wanpipe (Span) Number'?")
					== IDYES){

					Globals_New_Cfg.wanpipe_number = 
						(unsigned short)SendDlgItemMessage(hdlg, 
												COMBO_WANPIPE_NUMBER, CB_GETCURSEL, 0, 0);

					if(Globals_New_Cfg.wanpipe_number == 0){
						//Auto is selected. Restore to original AutoWanpipeNumber read from the Registry.
						Globals_New_Cfg.wanpipe_number = iAutoWanpipeNumber;
					}

					snprintf(Globals_New_Cfg.DeviceName, TMP_BUFFER_LEN, 
								"WANPIPE%d", Globals_New_Cfg.wanpipe_number);

					set_dialog_wanpipe_name(hdlg);

				}else{
					//make the current one selected
					SendDlgItemMessage(	hdlg, COMBO_WANPIPE_NUMBER, CB_SETCURSEL, 
										Globals_New_Cfg.wanpipe_number, 0);
				}
			}
			break;/* COMBO_WANPIPE_NUMBER */

		case COMBO_EXTERN_SYNCH_SOURCE:
			if (GET_WM_COMMAND_CMD(wParam, lParam) == CBN_SELCHANGE){
				//SynchSource: ask user to confirm, cause it is a serious change.
				if(yes_no_question("Are you sure you want change the 'Synchronization Source'?") == IDYES){

					Globals_New_Cfg.front_end_cfg.network_sync = 
						(unsigned short)SendDlgItemMessage(hdlg, COMBO_EXTERN_SYNCH_SOURCE, CB_GETCURSEL, 0, 0);


				}else{
					//make the current one selected
					SendDlgItemMessage(	hdlg, COMBO_EXTERN_SYNCH_SOURCE, CB_SETCURSEL, Globals_New_Cfg.front_end_cfg.network_sync, 0);
				}
			}
			break;/* COMBO_EXTERN_SYNCH_SOURCE */

		case IDCCHECK_START_PORT_WHEN_COMPUTER_STARTS:
			if (GET_WM_COMMAND_CMD(wParam, lParam) == BN_CLICKED){
				if(IsDlgButtonChecked(hdlg, IDCCHECK_START_PORT_WHEN_COMPUTER_STARTS)){
					Globals_New_Cfg.start_port_when_computer_starts = 1;
				}else{
					Globals_New_Cfg.start_port_when_computer_starts = 0;
				}
			}
			break;

		}//switch (LOWORD(wParam)) - process command notification
		break;
	}//switch (msg)
	return FALSE;
}							// PageDlgProc

///////////////////////////////////////////////////////////////////////////////

UINT CALLBACK DrvPageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p)
{
	if (msg == PSPCB_RELEASE && p->lParam)
	{
		Debug(DRV_CFG_PAGE, "PSPCB_RELEASE : DrvPageCallbackProc()\n");
	}
	return TRUE;	// matters only for PSPCB_CREATE
}

////////////////////////////////////////////////////////////////////////////////

int DriverOnNotify(
			HWND    ParentHwnd,
			LPNMHDR NmHdr,
			SETUPSTUFF * Params
		    )
{
    SP_DEVINSTALL_PARAMS spDevInstall = {0};
	BOOL	configuration_check_passed = FALSE;
    
    switch (NmHdr->code) 
	{
    case PSN_APPLY:
        //
        // Sent when the user clicks on Apply OR OK !!
        //
		configuration_check_passed = DriverSaveCfgData(ParentHwnd);

		Debug(DRV_CFG_PAGE, "DriverOnNotify(): PSN_APPLY");
		released_by_OK_button = 1;

		if(configuration_check_passed == TRUE)	return HANDLE_OK;
		else									return HANDLE_ERROR;

	case PSN_QUERYCANCEL:
		Debug(DRV_CFG_PAGE, "DriverOnNotify() : PSN_QUERYCANCEL");
		released_by_OK_button = 0;
		//fall through
    default:
		Debug(DRV_CFG_PAGE, "DriverOnNotify() : default case:");
        return HANDLE_DEFAULT;
    }
    return HANDLE_DEFAULT;   
} 

////////////////////////////////////////////////////////////////////////////////

void DriverInit(HKEY hkey, HWND hdlg)
{
	char			str_wp_num[TMP_BUFFER_LEN]; 
	unsigned int	ind, current_select_ind;
	
	///////////////////////////////////////////////////////////////////////
	//LogErrors
	///////////////////////////////////////////////////////////////////////
	if(Globals_New_Cfg.log_errors == 0){
		CheckDlgButton(hdlg, IDCHECK_IS_LOG_TO_FILE, BST_UNCHECKED);
	}else{
		CheckDlgButton(hdlg, IDCHECK_IS_LOG_TO_FILE, BST_CHECKED);
	}

	///////////////////////////////////////////////////////////////////////
	//WanpipeNumber
	///////////////////////////////////////////////////////////////////////
	SendDlgItemMessage(hdlg, COMBO_WANPIPE_NUMBER, CB_RESETCONTENT, 0, 0);
	ind = 0;
	while(ind < MAX_WP_NUMBER){
		if(ind == 0){
			snprintf(str_wp_num, TMP_BUFFER_LEN, "Auto"); 
		}else{
			snprintf(str_wp_num, TMP_BUFFER_LEN, "%d", ind); 

		}
		SendDlgItemMessage(	hdlg, COMBO_WANPIPE_NUMBER, CB_ADDSTRING, 0,
							(LPARAM)str_wp_num);
		ind++;
	}

	//make the current one selected
	SendDlgItemMessage(	hdlg, COMBO_WANPIPE_NUMBER, CB_SETCURSEL, 
						Globals_New_Cfg.wanpipe_number, 0);

	///////////////////////////////////////////////////////////////////////
	//network_sync
	///////////////////////////////////////////////////////////////////////
	SendDlgItemMessage(hdlg, COMBO_EXTERN_SYNCH_SOURCE, CB_RESETCONTENT, 0, 0);
	ind = 0;
	while(network_sync_options[ind].name != NULL){
		SendDlgItemMessage(	hdlg, COMBO_EXTERN_SYNCH_SOURCE, CB_ADDSTRING, 0, (LPARAM)network_sync_options[ind].name);
		ind++;
	}

	//make the current one selected
	SendDlgItemMessage(	hdlg, COMBO_EXTERN_SYNCH_SOURCE, CB_SETCURSEL, Globals_New_Cfg.front_end_cfg.network_sync, 0);

	///////////////////////////////////////////////////////////////////////
	//automatic port start
	///////////////////////////////////////////////////////////////////////
	if(Globals_New_Cfg.start_port_when_computer_starts == 1){
		CheckDlgButton(hdlg, IDCCHECK_START_PORT_WHEN_COMPUTER_STARTS, BST_CHECKED);
	}else{
		CheckDlgButton(hdlg, IDCCHECK_START_PORT_WHEN_COMPUTER_STARTS, BST_UNCHECKED);
	}

	///////////////////////////////////////////////////////////////////////
	//TDMV codec
	SendDlgItemMessage(hdlg, COMBO_TDMV_CODEC, CB_RESETCONTENT, 0, 0);
	ind = 0; current_select_ind = 0;
	while(aft_tdmv_codec_options[ind].name != NULL){
		SendDlgItemMessage(	hdlg, COMBO_TDMV_CODEC, CB_ADDSTRING, 0, (LPARAM)aft_tdmv_codec_options[ind].name);
		if(aft_tdmv_codec_options[ind].value == Globals_New_Cfg.front_end_cfg.tdmv_law){
			current_select_ind = ind;
		}
		ind++;
	}
	SendDlgItemMessage(hdlg, COMBO_TDMV_CODEC, CB_SETCURSEL, current_select_ind, 0);

	///////////////////////////////////////////////////////////////////////
	//HWEC options
//  { "HWEC_CLKSRC",				offsetof(wandev_conf_t, hwec_conf)+smemof(wan_hwec_conf_t, clk_src),			DTYPE_UINT},  
//  { "HWEC_PERSIST",				offsetof(wandev_conf_t, hwec_conf)+smemof(wan_hwec_conf_t, persist_disable),	DTYPE_UINT},  
//  { "TDMV_HWEC_PERSIST_DISABLE",  offsetof(wandev_conf_t, hwec_conf)+smemof(wan_hwec_conf_t, persist_disable),	DTYPE_UINT},  
//  { "HWEC_NOISE_REDUCTION",		offsetof(wandev_conf_t, hwec_conf)+smemof(wan_hwec_conf_t, noise_reduction),	DTYPE_UINT},  
//  { "HWEC_TONEDISABLERDELAY",		offsetof(wandev_conf_t, hwec_conf)+smemof(wan_hwec_conf_t, tone_disabler_delay), DTYPE_UINT},
	///////////////////////////////////////////////////////////////////////
	//HWEC_CLKSRC
	SendDlgItemMessage(hdlg, COMBO_HWEC_CLKSRC, CB_RESETCONTENT, 0, 0);
	ind = 0; current_select_ind = 0;
	while(hwec_clksrc_options[ind].name != NULL){
		SendDlgItemMessage(	hdlg, COMBO_HWEC_CLKSRC, CB_ADDSTRING, 0, (LPARAM)hwec_clksrc_options[ind].name);
		if(hwec_clksrc_options[ind].value == Globals_New_Cfg.wandev_conf.hwec_conf.clk_src){
			current_select_ind = ind;
		}
		ind++;
	}
	SendDlgItemMessage(hdlg, COMBO_HWEC_CLKSRC, CB_SETCURSEL, current_select_ind, 0);

	/////////////////////////////////////////////////////////////////////////////
	//tx tristate
	cfg_combobox_init(hdlg, COMBO_TX_TRISTATE, tx_tristate_mode, Globals_New_Cfg.front_end_cfg.tx_tristate_mode);
}

BOOL DriverSaveCfgData(HWND hdlg)
{
	char	local_tmp_buff[TMP_BUFFER_LEN]; 
	int		i;

	Debug(DRV_CFG_PAGE, "%s()\n", __FUNCTION__);

	///////////////////////////////////////////////////////////////////
	//Log to file setting
	///////////////////////////////////////////////////////////////////
	if(IsDlgButtonChecked(hdlg, IDCHECK_IS_LOG_TO_FILE))
	{	SetRegistryStringValue(hkey, "LogErrors", "1");
		Globals_New_Cfg.log_errors = 1;
	}else
	{	SetRegistryStringValue(hkey, "LogErrors", "0");
		Globals_New_Cfg.log_errors = 0;
	}

	///////////////////////////////////////////////////////////////////
	//WanpipeNumber
	///////////////////////////////////////////////////////////////////
	Globals_New_Cfg.wanpipe_number = 
		(unsigned short)SendDlgItemMessage(hdlg, COMBO_WANPIPE_NUMBER, CB_GETCURSEL, 0, 0);
	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.wanpipe_number);
	SetRegistryStringValue(hkey, "UserWanpipeNumber", local_tmp_buff);

	///////////////////////////////////////////////////////////////////
	//network_sync
	///////////////////////////////////////////////////////////////////
	Globals_New_Cfg.front_end_cfg.network_sync = 
		(unsigned short)SendDlgItemMessage(hdlg, COMBO_EXTERN_SYNCH_SOURCE, CB_GETCURSEL, 0, 0);
	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.front_end_cfg.network_sync);
	SetRegistryStringValue(hkey, "ExternalSynchronization", local_tmp_buff);

	///////////////////////////////////////////////////////////////////
	//Automatic Port start setting
	///////////////////////////////////////////////////////////////////
	if(Globals_New_Cfg.start_port_when_computer_starts == 1){
		SetRegistryStringValue(hkey, "StartPortWhenComputerStartsFlag", "1");
	}else{
		SetRegistryStringValue(hkey, "StartPortWhenComputerStartsFlag", "0");
	}
	///////////////////////////////////////////////////////////////////
	//TDMV codec
	i = SendDlgItemMessage(hdlg, COMBO_TDMV_CODEC, CB_GETCURSEL, 0, 0);
	if(CB_ERR != i){
		//set new value only if something is selected
		Globals_New_Cfg.front_end_cfg.tdmv_law = aft_tdmv_codec_options[i].value;
		SetRegistryStringValue(hkey, "tdmv_law", Globals_New_Cfg.front_end_cfg.tdmv_law);
	}
	///////////////////////////////////////////////////////////////////
	//HWEC_CLKSRC
	i = SendDlgItemMessage(hdlg, COMBO_HWEC_CLKSRC, CB_GETCURSEL, 0, 0);
	if(CB_ERR != i){
		//set new value only if something is selected
		Globals_New_Cfg.wandev_conf.hwec_conf.clk_src = hwec_clksrc_options[i].value;
		SetRegistryStringValue(hkey, "HWEC_CLKSRC", Globals_New_Cfg.wandev_conf.hwec_conf.clk_src);
	}

	/////////////////////////////////////////////////////////////////////
	Globals_New_Cfg.front_end_cfg.tx_tristate_mode =
		cfg_combobox_get_value(hdlg, COMBO_TX_TRISTATE, tx_tristate_mode);
	SetRegistryStringValue(hkey, "FE_TXTRISTATE", Globals_New_Cfg.front_end_cfg.tx_tristate_mode);

	return TRUE;
}

void restart_device()
{
	HANDLE	hRestartWaitHandle;
	HWND	thewin;
	int		err = 0;
	// Set the PropChangeParams structure.
	SP_PROPCHANGE_PARAMS	pcp;
	
	Debug(DRV_CFG_PAGE, "********************restart_device()*********************\n");

	thewin = CreateDialog(	hInst, MAKEINTRESOURCE(DIALOG_RESTARTING),
							GetActiveWindow(), (DLGPROC)RestartingDriverDlg);
	if(thewin == NULL){
		err = 1;
		goto error;
	}
	hRestartWaitHandle = CreateEvent( NULL, FALSE, FALSE, "devchangeevent"); 

	memset(&pcp, 0x00, sizeof(SP_PROPCHANGE_PARAMS));

	pcp.ClassInstallHeader.cbSize = sizeof(SP_CLASSINSTALL_HEADER);
	pcp.ClassInstallHeader.InstallFunction = DIF_PROPERTYCHANGE;


	//1-st disable our device
	pcp.Scope		= DICS_FLAG_CONFIGSPECIFIC;
	pcp.StateChange = DICS_DISABLE;
	pcp.HwProfile	= 0;

	if (!SetupDiSetClassInstallParams(stuff->info, stuff->did,
			(SP_CLASSINSTALL_HEADER *)&pcp,	sizeof(pcp))){

		Debug(DRV_CFG_PAGE, "DICS_DISABLE: SetupDiSetClassInstallParams() failed!\n");
		err = 1;
		goto error;
	}

	if (!SetupDiCallClassInstaller(DIF_PROPERTYCHANGE, stuff->info, stuff->did)){
		Debug(DRV_CFG_PAGE, "DICS_DISABLE: SetupDiCallClassInstaller() failed!\n");
		err = 2;
		goto error;
	}

	//wait for device to be stopped, 2 seconds should be enough.
	WaitForSingleObject( hRestartWaitHandle, 2*1000);

	//2-nd enable our device
	pcp.Scope		= DICS_FLAG_CONFIGSPECIFIC;
	pcp.StateChange = DICS_ENABLE;
	pcp.HwProfile	= 0;

	if (!SetupDiSetClassInstallParams(stuff->info, stuff->did,
		(SP_CLASSINSTALL_HEADER *)&pcp,	sizeof(pcp))){

		Debug(DRV_CFG_PAGE, "DICS_ENABLE: SetupDiSetClassInstallParams() failed!\n");
		err = 3;
		goto error;
	}

	if (!SetupDiCallClassInstaller(DIF_PROPERTYCHANGE, stuff->info, stuff->did)){
		Debug(DRV_CFG_PAGE, "DICS_ENABLE: SetupDiCallClassInstaller() failed!\n");
		err = 4;
		goto error;
	}

	//Wait for 2 seconds while device is getting restarted.
	//It takes A200/A400/B800 Analog cards the longest time to start comparing to other cards.
	WaitForSingleObject( hRestartWaitHandle,  2*1000);

error:
	if(err){
		LPVOID lpMsgBuf;
		FormatMessage( 
				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				FORMAT_MESSAGE_FROM_SYSTEM | 
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				GetLastError(),
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
				(LPTSTR) &lpMsgBuf,
				0,
				NULL );
		// Process any inserts in lpMsgBuf.
		// ...
		// Display the string.
		MessageBox( GetActiveWindow(), (LPCTSTR)lpMsgBuf, "Error",
			MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL);
		// Free the buffer.
		LocalFree( lpMsgBuf );
	}//if(err)

	DestroyWindow(thewin);
}


static LRESULT CALLBACK RestartingDriverDlg(HWND hdlg, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	switch(iMessage) {
	case WM_SYSCOMMAND:
		switch(wParam & 0xfff0) {
		case SC_CLOSE:	/* Don't allow user to close window this way */
			return(1);
		}
		break;
	case WM_INITDIALOG:
		return(1L);
	}
	return(0);
}

/*++

Routine Description:

    1.Check if the zero terminated string contains only numerical
	  characters and is not empty.
	2.If the check passed, return the integer value of the number in
	  the buffer.

Arguments:

    buff - The pointer to the string.

Return Value:

	1.	-1 if one of the checks failes.
	Or
	2.	the value of the number in the string.
	
--*/
int is_number(char * buff)
{
	unsigned int i;
	char		 *tmp = buff;

	if(!strlen(buff))
	{	Debug(DRV_CFG_PAGE, "is_number: the string is empty!!");
		return -1;
	}

	for(i = 0; i < strlen(buff); i++)
	{	if(!isdigit(*(tmp++)))
		{	return -1;
		}
	}
	Debug(DRV_CFG_PAGE, "is_number returns: %d", atoi(buff));
	return atoi(buff);
}

/////////////////////////////////////////////////////////////////////////////////////
//Wanpipe name must be the same on ALL pages
void set_dialog_wanpipe_name(HWND hdlg)
{
	char	local_tmp_buff[TMP_BUFFER_LEN]; 

	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "Device Name: %s", Globals_New_Cfg.DeviceName);

	SetDlgItemText(hdlg, IDC_SAMNAME, local_tmp_buff);
}

/////////////////////////////////////////////////////////////////////////////////////
